;;; Studio Tecnico Arch. Giuseppe Conte  
;;; via Roma, 28
;;; 72026 - San Pancrazio Salentino (BR) - Italy
;;;
;;; Plugin  : draw-grid.scm
;;; Author  : Arch. Giuseppe Conte 
;;; Date    : 27 settembre 2002 - San Pancrazio
;;; Revision: 11 giugno 2006
;;; 					10 dicembre 2007 update to TinyScheme
;;;						18 ottobre 2008 update to GIMP 2.6
;;; Version : 2.4.1 - 2.4.2
;;; Last version at: http://xoomer.alice.it/lwcon/gimp/script-fu/draw-grid.htm
;;; Help guide at  : http://xoomer.alice.it/lwcon/gimp/script-fu/draw-grid.htm
;;; Required : Gimp 2.4.1 or last
;;;
;;; Description: 
;;; Draw grid into image.
;;; -----------------------------------------------------------------------------
;;;
;;; License:
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;; 
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;; 
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;;; 
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;; 

;;; Define the function:

(define (script-fu-draw-grid image inLayer dx dy angle tipo)

	(let* (	(oriz 0)
					(vert 0)
					(width 0)
					(height 0)
					(segment (make-vector 4))
					(alfa 0)
					(outx 0)
					(outy 0)
					(qx 0)
					(qy 0)
					(ks 0)
					(kd 0)
					(cnt 0)
					(xa 0)
					(ya 0)
					(xb 0)
					(yb 0)
					(npoint 4)
				)


;tipo di griglia
;;;griglia
	(if (= tipo 0) 
		(begin 
			(set! oriz 0)
			(set! vert 0)
		);begin
		;(set! oriz 1)
	);end if
;;;solo righe orizzontali
	(if (= tipo 1) 
		(begin 
			(set! oriz 0)
			(set! vert 1)
		);begin
		;(set! oriz 0)
	);end if
;;;solo righe verticali
	(if (= tipo 2) 
		(begin 
			(set! oriz 1)
			(set! vert 0)
		);begin
		;(set! oriz 0)
	);end if
		

;determina le dimensioni dell'immagine	
	(set! width (car (gimp-image-width image)))
	(set! height (car (gimp-image-height image)))
	
;imposta un array per contenere le coordinate
	(set! npoint 4)
	(set! segment (make-vector 4))  

;imposta l'angolo in radianti
	(set! alfa (* angle (/ *pi* 180)))	
	
	(set! outy (* width (tan alfa)))
	(set! outx (* width (tan alfa)))
	
;calcola l'incremento verticale ed orizzontale
	(set! qx (* dx (cos alfa)))
	(set! qy (* dy (cos alfa)))	
	
;imposta i prolungamenti sinistro e destro
	(set! ks (+ height outy))
	(set! kd (- 0 outy))
	(set! cnt 0)
	
;coordinate iniziali x
	(set! xa 0)
	(set! xb width)	
	(set! ya 0)
	(set! yb kd)
	
;(gimp-message "sono qui ")

;inizio delle operazioni che potranno essere annullate con un solo undo
	(gimp-undo-push-group-start image)

;tracciamento line orizzontali

  (if (= oriz 0)
  	(begin	
	(while (<= cnt ks)
				
		(set! ya (+ ya qy))
		(set! yb (+ yb qy))
		
  		(vector-set! segment 0 (* 1 xa))
  		(vector-set! segment 1 (* 1 ya))
  		(vector-set! segment 2 (* 1 xb))
  		(vector-set! segment 3 (* 1 yb))
  	       
		(gimp-pencil inLayer npoint segment)
				
		(set! cnt (+ cnt qy))
		
   	);end while
   	);end begin
   );end if
   
;tracciamento linee verticali

	(set! kd (+ width outx))
	(set! ks (- 0 outx))
	(set! cnt 0)
	
;coordinate iniziali x
	(set! xa ks)
	(set! xb 0)	
	(set! ya 0)
	(set! yb height)

  (if (= vert 0)
  	(begin
	(while (<= cnt kd)
				
		(set! xa (+ xa qx))
		(set! xb (+ xb qx))
		
  		(vector-set! segment 0 (* 1 xa))
  		(vector-set! segment 1 (* 1 ya))
  		(vector-set! segment 2 (* 1 xb))
  		(vector-set! segment 3 (* 1 yb))
  	       
		(gimp-pencil inLayer npoint segment)
				
		(set! cnt (+ cnt qx))
		
   	);end while  
	);end begin
    );end if	

;fine delle operazioni che potranno essere annullate con un solo undo
	(gimp-undo-push-group-end image)    	
	
  	  (gimp-displays-flush)
      
       
	);;let
) ;;def


(script-fu-register
 "script-fu-draw-grid"
 _"<Image>/_ATG/_Draw/Draw _Grid"
 "To draw grid or alone inclined parallel line."
 "Arch. Giuseppe Conte <http://xoomer.alice.it/lwcon/>"
 "2008, Giuseppe Conte"
 "18 ottobre 2008 - 72026 San Pancrazio Salentino (BR) - Italy"
 "RGB* GRAY* INDEXED*"
 SF-IMAGE "The Image" 0
 SF-DRAWABLE "The Layer" 0
 SF-ADJUSTMENT "Delta x" '(20 0 9999 1 10 0 1)
 SF-ADJUSTMENT "Delta y" '(20 0 9999 1 10 0 1)
 SF-ADJUSTMENT "Angle" '(45 0 89.9999 1 10 0 1)
 SF-OPTION "Tipo:" '("Griglia" "Line Orizzontali" "Line Verticali")
 
)
