; +--------------------------------------------------------------------------+
; +--------------------------------------------------------------------------+
; | GIMP - The GNU Image Manipulation Program                                |
; | Copyright (C) 1995 Spencer Kimball and Peter Mattis                      |
; |                                                                          |
; | This program is free software; you can redistribute it and/or modify     |
; | it under the terms of the GNU General Public License as published by     |
; | the Free Software Foundation; either version 2 of the License, or        |
; | (at your option) any later version.                                      |
; |                                                                          |
; | This program is distributed in the hope that it will be useful,          |
; | but WITHOUT ANY WARRANTY; without even the implied warranty of           |
; | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            |
; | GNU General Public License for more details.                             |
; |                                                                          |
; | You should have received a copy of the GNU General Public License        |
; | along with this program; if not, write to the Free Software              |
; | Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                |
; +--------------------------------------------------------------------------+
; +--------------------------------------------------------------------------+
; | Remove Holes (from Selection)                                            |
; +--------------------------------------------------------------------------+
; | remove-holes-f-s.scm - version 1.02                                      |
; | Copyright (C) 2010 Gino D                                                |
; | ginodonig@gmail.com                                                      |
; +--------------------------------------------------------------------------+
; | With this script you can eliminate the holes of a selection, if by hole  |
; | we mean every unselected or partially selected area which is surrounded  |
; | by a fully selected area. The script can optionally apply some initial   |
; | adjustments to the selection in order to globally improve the action of  |
; | removal, and it allows to plan a final feathering of the selection as    |
; | well.                                                                    |
; +--------------------------------------------------------------------------+
; | VERSION HISTORY                                                          |
; |                                                                          |
; | 1.00 - January 2010                                                      |
; |  * First release.                                                        |
; |                                                                          |
; | 1.01 - January 2010                                                      |
; |  * Added the ability to finally feather the selection.                   |
; |                                                                          |
; | 1.02 - July 2010                                                         |
; |  * Only reshaped and cleaned up the code.                                |
; +--------------------------------------------------------------------------+
; +--------------------------------------------------------------------------+

(define
	(script-fu-remove-holes-f-s
		I D ns ats thr frs rof
	)
	(let*
		(
			(D_il (car (gimp-drawable-is-layer D)))
			(wt (car (gimp-image-width I)))
			(ht (car (gimp-image-height I)))
			(D_ilm (car (gimp-drawable-is-layer-mask D)))
			(thr (truncate (+ thr 0.5)))
		)
		(gimp-context-push)
		(gimp-image-undo-group-start I)
			(cond ((= (car (gimp-selection-is-empty I)) 1)
				(gimp-image-undo-group-end I)
				(gimp-displays-flush)
				(gimp-context-pop)
				(quit)
			))
			(if (= D_ilm 1)
				(set! D (car (gimp-layer-from-mask D)))
			)
			(gimp-image-resize I (+ wt 2) (+ ht 2) 1 1)
			(define C0 (car (gimp-selection-save I)))
			(gimp-selection-none I)
			(if (= ns 1)
				(plug-in-normalize 1 I C0)
			)
			(define C1 (car(gimp-channel-copy C0)))
			(gimp-image-add-channel I C1 -1)
			(if (= ats 0)
				(gimp-threshold C1 255 255)
				(gimp-threshold C1 thr 255)
			)
			(gimp-fuzzy-select C1 0 0 0 2 0 0 0 0)
			(gimp-selection-invert I)
			(if (= ats 0)
				(gimp-selection-combine C0 0)
			)
			(if (= frs 1)
				(gimp-selection-feather I rof)
			)
			(gimp-image-remove-channel I C1)
			(gimp-image-remove-channel I C0)
			(gimp-image-resize I wt ht -1 -1)
			(if (= D_il 1)
				(gimp-image-set-active-layer I D)
				(gimp-image-set-active-channel I D)
			)
		(gimp-image-undo-group-end I)
		(gimp-displays-flush)
		(gimp-context-pop)
	)
)
(script-fu-register
	"script-fu-remove-holes-f-s"
	"Remove Holes..."
	"Remove holes from the current selection"
	"Gino D"
	"Gino D"
	"2010"
	"*"
	SF-IMAGE     		"Image"                                		0
	SF-DRAWABLE  		"Drawable"                             		0
	SF-TOGGLE    		"Normalize selection at first"         		0
	SF-TOGGLE    		"Apply threshold to selection at first"		0
	SF-ADJUSTMENT		"White threshold"                      		'(128 0 255 1 1 0 0)
	SF-TOGGLE    		"Feather resultant selection"          		0
	SF-ADJUSTMENT		"Radius of feather (in pixels)"        		'(5 0 32767 1 1 3 1)
)
(script-fu-menu-register
	"script-fu-remove-holes-f-s"
	"<Image>/Select"
)